package com.loafle.overflow.container.general.crawler.impl.database;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.crawler.Crawler;
import com.loafle.overflow.model.sensorconfig.Item;
import com.loafle.overflow.model.sensorconfig.ResultSet;
import com.loafle.overflow.model.sensorconfig.ResultSetRow;
import com.loafle.overflow.model.sensorconfig.SensorConfig;

public abstract class DatabaseCrawler implements Crawler {

  public DatabaseCrawler() {
  }

  @Override
  public void auth(Map<String, Object> authInfoMap) throws OverflowException {
    Connection conn = null;

    String url = (String) authInfoMap.get("url");
    String id = (String) authInfoMap.get("id");
    String pw = (String) authInfoMap.get("pw");

    try {
      loadDriverClass();
    } catch (ClassNotFoundException e) {
      throw new OverflowException("DatabaseCrawler", e);
    }

    try {
      conn = java.sql.DriverManager.getConnection(url, id, pw);
    } catch (Exception e) {
      throw new OverflowException("DatabaseCrawler", e);
    } finally {
      if (conn != null) {
        try {
          conn.close();
        } catch (SQLException e) {
          e.printStackTrace();
        }
      }
    }
  }

  @Override
  public Map<String, String> get(SensorConfig sensorConfig) throws OverflowException {
    Connection conn = null;
    Statement stmt = null;

    java.sql.ResultSet rs = null;

    String url = (String) sensorConfig.getTarget().getAuth().get("url");
    String id = (String) sensorConfig.getTarget().getAuth().get("id");
    String pw = (String) sensorConfig.getTarget().getAuth().get("pw");

    try {
      loadDriverClass();
    } catch (ClassNotFoundException e) {
      throw new OverflowException("DatabaseCrawler", e);
    }

    Map<String, String> resultMap = new HashMap<>();

    try {
      conn = java.sql.DriverManager.getConnection(url, id, pw);
      stmt = conn.createStatement();

      for (Item item : sensorConfig.getItems()) {

        ResultSet resultSet = ResultSetRow.newInstance(item);
        Map<String, Integer> meta = resultSet.getMeta();

        rs = stmt.executeQuery((String) item.getQueryInfo().getQuery());

        while (rs.next()) {
          List<String> row = new ArrayList<>(Arrays.asList(new String[meta.size()]));
          for (Map.Entry<String, Integer> info : meta.entrySet()) {
            String data = rs.getString(info.getKey());
            data = data.trim();
            row.set(info.getValue().intValue(), data);
          }
          resultSet.addRow(row);
        }

        rs.close();
        resultMap.putAll(resultSet.getData());
      }
      return resultMap;

    } catch (Exception e) {
      throw new OverflowException("DatabaseCrawler", e);
    } finally {
      if (conn != null) {
        try {
          conn.close();
        } catch (SQLException e) {
          e.printStackTrace();
        }
      }
      if (rs != null) {
        try {
          rs.close();
        } catch (SQLException e) {
          e.printStackTrace();
        }
      }
      if (stmt != null) {
        try {
          stmt.close();
        } catch (SQLException e) {
          e.printStackTrace();
        }
      }
    }
  }

  protected abstract void loadDriverClass() throws ClassNotFoundException;
}
