package com.loafle.overflow.container.general.crawler.impl.mongodb;

import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.crawler.Crawler;
import com.loafle.overflow.model.sensorconfig.Item;
import com.loafle.overflow.model.sensorconfig.ResultSet;
import com.loafle.overflow.model.sensorconfig.SensorConfig;
import com.mongodb.MongoClient;
import com.mongodb.MongoClientOptions;
import com.mongodb.ServerAddress;
import com.mongodb.client.MongoDatabase;
import org.bson.Document;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class MongoDBCrawler implements Crawler {
  @Override
  public String name() {
    return "MONGODB";
  }

  @Override
  public String toString() {
    return "MongoDB Crawler";
  }

  @Override
  public void auth(Map<String, Object> authInfoMap) throws OverflowException {
    throw new OverflowException("Not implemented", null);
  }

  @Override
  public Map<String, String> get(SensorConfig sensorConfig) throws OverflowException {
    MongoClient mongoClient = null;
    Map<String, String> returnMap = new HashMap<String, String>();

    String dataBaseName = null;
    String statusCommand = null;

    String targetIP = sensorConfig.getTarget().getConnection().getIp();
    int targetPort = sensorConfig.getTarget().getConnection().getPort();

    try {

      mongoClient = new MongoClient(new ServerAddress(targetIP, targetPort),
          MongoClientOptions.builder().serverSelectionTimeout(1000).connectTimeout(1000).socketTimeout(1000).build());

      ResultSet rs = null;
      List<Item> items = sensorConfig.getItems();
      List<String> row = null;
      for (Item item : items) {

        rs = ResultSet.newInstance(item);
        Map<String, Integer> metaMap = rs.getMeta();
        row = new ArrayList<>(Arrays.asList(new String[metaMap.size()]));

        dataBaseName = (String) item.getQueryInfo().getExtend().get("dataBaseName");
        statusCommand = (String) item.getQueryInfo().getExtend().get("statusCommand");

        MongoDatabase database = mongoClient.getDatabase(dataBaseName);
        Document serverStatus = database.runCommand(new Document(statusCommand, 1));

        Map<String, String> re = (Map<String, String>) serverStatus.get(item.getQueryInfo().getQuery());

        for (Map.Entry<String, Integer> info : metaMap.entrySet()) {
          row.set(info.getValue().intValue(), String.valueOf(re.get(info.getKey())));
        }

        rs.addRow(row);
        returnMap.putAll(rs.getData());
      }
    } catch (Exception e) {
      throw new OverflowException("MongoDBCrawler", e);
    } finally {
      if (mongoClient != null) {
        mongoClient.close();
      }
    }
    return returnMap;
  }
}
