package com.loafle.overflow.container.general.crawler.impl.redis;

import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.crawler.Crawler;
import com.loafle.overflow.model.sensorconfig.Item;
import com.loafle.overflow.model.sensorconfig.ResultSet;
import com.loafle.overflow.model.sensorconfig.SensorConfig;

import redis.clients.jedis.Jedis;

import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;

public class RedisCralwer implements Crawler {

  private static final Logger LOGGER = Logger.getLogger(RedisCralwer.class.getName());

  public RedisCralwer() {

  }

  @Override
  public String name() {
    return "REDIS";
  }

  @Override
  public String toString() {
    return "Redis Crawler";
  }

  @Override
  public void auth(Map<String, Object> authIMap) throws OverflowException {
    throw new OverflowException("Not implemented", null);
  }

  @Override
  public Map<String, String> get(SensorConfig sensorConfig) throws OverflowException {
    String targetIP = sensorConfig.getTarget().getConnection().getIp();
    int targetPort = sensorConfig.getTarget().getConnection().getPort();

    String authpw = (String) sensorConfig.getTarget().getAuth().get("authpw");

    Jedis jedis = null;
    List<Item> items = sensorConfig.getItems();

    try {
      jedis = new Jedis(targetIP, targetPort, false);
      if (authpw != null && !authpw.equals("")) {
        String code = jedis.auth(authpw);
        if (!code.equals("")) {
          // set error auth
        }
      }

      Map<String, String> returnMap = new HashMap<String, String>();

      for (Item item : items) {
        ResultSet resultSet = ResultSet.newInstance(item);

        String queryString = (String) item.getQueryInfo().getQuery();
        String info = jedis.info(queryString);
        Map<String, String> resultMap = (Map<String, String>) parseToMap(info).get(queryString);

        resultSet.addRow(parse(resultMap, resultSet.getMeta()));
        
        returnMap.putAll(resultSet.getData());
      }

      return returnMap;
    } catch (Exception e) {
      throw new OverflowException("RedisCralwer", e);
    } finally {
      if (jedis != null) {
        jedis.close();
      }
    }
  }

  private List<String> parse(Map<String, String> dataMap, Map<String, Integer> meta) {

    List<String> row = new ArrayList<>(Arrays.asList(new String[meta.size()]));
    for (Map.Entry<String, Integer> info : meta.entrySet()) {
      String data = dataMap.get(info.getKey());
      data = data.trim();
      row.set(info.getValue().intValue(), data);
    }
    return row;
  }

  public Map<String, Object> parseToMap(String source) {
    String[] categorys = source.split("\r\n\r\n");
    Map<String, Object> result = new HashMap<String, Object>();

    for (String category : categorys) {

      if (category.length() == 0) {
        continue;
      }
      Map<String, String> sm = new HashMap<String, String>();
      String[] lines = category.split("\r\n");

      int idx = 0;
      for (String line : lines) {

        if (idx == 0) {
          idx++;
          continue;
        }
        if (line.length() == 0) {
          continue;
        }

        String[] items = line.split(":");
        if (items.length != 0 && items.length > 1) {
          sm.put(items[0], items[1]);
        }
      }
      String[] ca = lines[0].split(" ");
      result.put(ca[1], sm);
    }
    return result;
  }

}
