package com.loafle.overflow.container.general.crawler.impl.snmp;

import com.loafle.overflow.crawler.Crawler;
import com.loafle.overflow.model.sensorconfig.Connection;
import com.loafle.overflow.model.sensorconfig.Item;
import com.loafle.overflow.model.sensorconfig.ResultSet;
import com.loafle.overflow.model.sensorconfig.SensorConfig;
import com.loafle.overflow.container.general.crawler.impl.snmp.version.SNMPv2c;
import com.loafle.overflow.container.general.crawler.impl.snmp.version.SNMPv3;
import com.loafle.overflow.core.exception.OverflowException;

import org.snmp4j.Snmp;
import org.snmp4j.TransportMapping;
import org.snmp4j.smi.Address;
import org.snmp4j.transport.DefaultUdpTransportMapping;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class SNMPCrawler implements Crawler {
  @Override
  public String name() {
    return "SNMP";
  }

  @Override
  public String toString() {
    return "SNMP Crawler";
  }

  @Override
  public void auth(Map<String, Object> authIMap) throws OverflowException {
    throw new OverflowException("Not implemented", null);
  }

  @Override
  public Map<String, String> get(SensorConfig sensorConfig) throws OverflowException {
    Snmp snmp = null;
    TransportMapping<? extends Address> transport = null;

    Map<String, String> returnMap = new HashMap<String, String>();

    try {
      transport = new DefaultUdpTransportMapping();
      snmp = new Snmp(transport);
      transport.listen();

      List<Item> items = sensorConfig.getItems();
      String[] oids = null;

      Map<String, String> snmpResultMap = null;
      for (Item item : items) {
        oids = new String[item.getKeys().size()];
        for (int i = 0; i < item.getKeys().size(); i++) {
          oids[i] = item.getKeys().get(i).getKey();
        }

        snmpResultMap = getSNMPV(snmp, sensorConfig, oids);
        returnMap.putAll(convertResultSet(snmpResultMap, item).getData());
      }
      return returnMap;

    } catch (Exception e) {
      throw new OverflowException("RedisCralwer", e);
    } finally {
      if (snmp != null) {
        try {
          snmp.close();
        } catch (IOException e) {
          e.printStackTrace();
        }
      }
      if (transport != null) {
        try {
          transport.close();
        } catch (IOException e) {
          e.printStackTrace();
        }
      }
    }
  }

  private Map<String, String> getSNMPV(Snmp snmp, SensorConfig sensorConfig, String[] oids) throws Exception {

    String targetVer = (String) sensorConfig.getTarget().getAuth().get("version");
    switch (targetVer) {
    case "v2c":
      return getV2c(snmp, sensorConfig, oids);
    case "v3":
      return getV3(snmp, sensorConfig, oids);
    default:
      new Exception("Unknown SNMP protocol : " + targetVer).printStackTrace();
    }

    return null;
  }

  private Map<String, String> getV2c(Snmp snmp, SensorConfig sensorConfig, String[] oids) throws Exception {

    SNMPv2c snmpV2 = new SNMPv2c(snmp);

    Map<String, Object> authInfo = sensorConfig.getTarget().getAuth();
    Connection connInfo = sensorConfig.getTarget().getConnection();

    String ip = connInfo.getIp();
    int port = connInfo.getPort();
    String community = (String) authInfo.get("community");

    return snmpV2.get(ip, port, community, oids);
  }

  private Map<String, String> getV3(Snmp snmp, SensorConfig sensorConfig, String[] oids) throws Exception {
    SNMPv3 snmpV3 = new SNMPv3(snmp);

    Map<String, Object> authInfo = sensorConfig.getTarget().getAuth();
    Connection connInfo = sensorConfig.getTarget().getConnection();

    String ip = connInfo.getIp();
    int port = connInfo.getPort();
    //            String method = sensorConfig.get;

    String user = (String) authInfo.get("user");
    String authType = (String) authInfo.get("authType");
    String authPass = (String) authInfo.get("authPass");
    String privType = (String) authInfo.get("privType");
    String privPass = (String) authInfo.get("privPass");

    return snmpV3.get(ip, port, user, authType, authPass, privType, privPass, oids);
  }

  protected ResultSet convertResultSet(Map<String, String> map, Item item) {

    ResultSet rs = ResultSet.newInstance(item);

    List<String> row = new ArrayList<>(Arrays.asList(new String[map.size()]));

    Map<String, Integer> metaMap = rs.getMeta();

    for (Map.Entry<String, Integer> info : metaMap.entrySet()) {
      row.set(info.getValue().intValue(), map.get(info.getKey()));
    }
    rs.addRow(row);

    return rs;
  }

}
